# Copyright 2021- Le Filament (https://le-filament.com)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html)

from odoo import _, fields, models
from odoo.exceptions import UserError
from odoo.osv import expression

from odoo.addons.api_connector.tools.date_utils import local_to_utc


class AccOperationWizard(models.TransientModel):
    _name = "acc.operation.wizard"
    _description = "Récupération des courbes pour une date donnée"

    # ------------------------------------------------------
    # Fields declaration
    # ------------------------------------------------------
    def _default_operation_id(self):
        op = self.env["acc.operation"].browse(
            self.env.context.get("active_ids")[0]
            if self.env.context.get("active_ids")
            else []
        )
        return op

    operation_id = fields.Many2one(
        "acc.operation", default=lambda self: self._default_operation_id()
    )
    start_date = fields.Date("Date de début")
    end_date = fields.Date("Date de fin (exclue)")
    prm_cons_ids = fields.Many2many(
        "acc.counter",
        relation="acc_counter_cons_rel",
        column1="cons_id",
        column2="op_id",
        string="PRM de soutirage",
    )
    prm_prod_ids = fields.Many2many(
        "acc.counter",
        relation="acc_counter_prod_rel",
        column1="prod_id",
        column2="op_id",
        string="PRM d'injection",
    )

    # ------------------------------------------------------
    # SQL Constraints
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Default methods
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Computed fields / Search Fields
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Onchange / Constraints
    # ------------------------------------------------------

    # ------------------------------------------------------
    # CRUD methods (ORM overrides)
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Actions
    # ------------------------------------------------------
    def get_curves(self):
        if not self.start_date or not self.end_date:
            raise UserError(
                _("Les champs Date de début et Date de fin sont obligatoires")
            )
        if (self.end_date - self.start_date).days > 31:
            raise UserError(_("L'intervalle de temps ne doit pas dépasser 31 Jours"))

        if self.start_date >= self.end_date:
            raise UserError(_("La date de fin doit être supérieure à la date de début"))

        # Instanciation des domaines de recherche
        domain_base = [
            ("acc_operation_id", "=", self.operation_id.id),
        ]
        domain_extra = []
        domain_extra_cdc = []
        if self.prm_cons_ids:
            domain_cons = [
                ("acc_counter_id", "in", self.prm_cons_ids.ids),
            ]
            domain_cons_cdc = expression.AND(
                [domain_cons, [("comp_data_type", "in", ("autocons", "cons"))]]
            )
            domain_extra = domain_cons
            domain_extra_cdc = domain_cons_cdc
        if self.prm_prod_ids:
            domain_prod = [
                ("acc_counter_id", "in", self.prm_prod_ids.ids),
            ]
            domain_prod_cdc = expression.AND(
                [domain_prod, [("comp_data_type", "in", ("prod", "surplus"))]]
            )
            domain_extra = domain_prod
            domain_extra_cdc = domain_prod_cdc
        if self.prm_cons_ids and self.prm_prod_ids:
            domain_extra = expression.OR([domain_cons, domain_prod])
            domain_extra_cdc = expression.OR([domain_cons_cdc, domain_prod_cdc])

        domain_periods = expression.AND([domain_base, domain_extra])
        start_datetime = local_to_utc(self.start_date, "Europe/Paris")
        end_datetime = local_to_utc(self.end_date, "Europe/Paris")
        domain_date = [
            ("date_slot", ">=", start_datetime),
            ("date_slot", "<", end_datetime),
        ]
        domain_cdc = expression.AND([domain_base, domain_extra_cdc, domain_date])

        # Vérification que des périodes existent pour cet intervalle de date
        period_ids = self.env["acc.counter.period"]._get_periods_from_interval(
            domain_periods,
            self.start_date,
            self.end_date,
        )
        if not period_ids:
            raise UserError(
                _("Aucun période trouvée pour le(s) PRM(s) et la période demandés")
            )

        # Vérification si des données existent déjà
        # pour cet intervalle de date
        rec_ids = self.env["acc.enedis.cdc"].search(domain_cdc)
        if rec_ids:
            wizard = self.env["acc.operation.wizard.confirm"].create(
                {
                    "operation_id": self.operation_id.id,
                    "start_date": self.start_date,
                    "end_date": self.end_date,
                    "prm_cons_ids": self.prm_cons_ids,
                    "prm_prod_ids": self.prm_prod_ids,
                    "rec_ids": rec_ids,
                    "mess": _(
                        "Des données existent déjà pour cette période. Etes-vous"
                        " sûr de vouloir écraser les données existantes ?"
                    ),
                }
            )
            return {
                "name": "Confirmation récupération des données",
                "type": "ir.actions.act_window",
                "view_mode": "form",
                "res_model": "acc.operation.wizard.confirm",
                "res_id": wizard.id,
                "target": "new",
            }
        else:
            self.env["acc.enedis.cdc"]._curves(
                self.operation_id,
                self.start_date,
                self.end_date,
                self.prm_cons_ids,
                self.prm_prod_ids,
            )
            return {"type": "ir.actions.act_window_close"}

    def get_curves_all(self):
        if not self.operation_id.date_start_contract:
            raise UserError(
                _(
                    "Renseigner une date de début de contrat pour"
                    " pouvoir récupérer les courbes à partir de cette date."
                )
            )

        rec_ids = self.env["acc.enedis.cdc"].search(
            [("acc_operation_id", "=", self.operation_id.id)]
        )
        if rec_ids:
            wizard = self.env["acc.operation.wizard.confirm"].create(
                {
                    "operation_id": self.operation_id.id,
                    "prm_cons_ids": self.prm_cons_ids,
                    "prm_prod_ids": self.prm_prod_ids,
                    "rec_ids": rec_ids,
                    "mess": _(
                        "Des données existent pour cette période. Etes-vous"
                        " sûr de vouloir écraser toutes les données existantes?"
                    ),
                }
            )
            return {
                "name": "Confirmation récupération des données",
                "type": "ir.actions.act_window",
                "view_mode": "form",
                "res_model": "acc.operation.wizard.confirm",
                "res_id": wizard.id,
                "target": "new",
            }
        else:
            self.env["acc.enedis.cdc"]._get_curves_all(
                self.operation_id, self.prm_cons_ids, self.prm_prod_ids
            )
            return {"type": "ir.actions.act_window_close"}

    # ------------------------------------------------------
    # Business methods
    # ------------------------------------------------------
